@@ -1,5 +1,7 @@
+#pragma once
+
 /*
- * Copyright (c) 2019 - 2020 Andri Yngvason
+ * Copyright (c) 2022 Andri Yngvason
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,16 +16,4 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#pragma once
-
-#include <stdio.h>
-
-#ifdef NDEBUG
-#define log_debug(...)
-#else
-#define log_debug(fmt, ...) \
-	fprintf(stderr, "%s:%d: " fmt, __FILE__, __LINE__, ## __VA_ARGS__)
-#endif
-
-#define log_error(...) \
-	fprintf(stderr, "ERROR: " __VA_ARGS__)
+void nvnc__log_init(void);
